//
// Benzaiten - A Simple Game Framework.
// Copyright (C) 2008, 2009 Geisha Studios.
//
// This library is free software;  you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This library is distributed in hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
// License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//
#if defined (HAVE_BENZAITEN_CONFIG_H)
#include <benzaiten_config.h>
#endif // HAVE_BENZAITEN_CONFIG_H
#include "Settings.hpp"

#if defined (WIN32)
#include "SettingsImplWin32.hpp"
#elif defined (APPLE)
#include "SettingsImplOSX.hpp"
#elif defined (UNIX)
#include "SettingsImplUnix.hpp"
#else
#error "This platform does not have a low level settings implementation."
#endif

using namespace benzaiten;

Settings::Settings (const std::string &appName):
#if defined (WIN32)
    impl_ (new SettingsImplWin32 (appName))
#elif defined (APPLE)
    impl_ (new SettingsImplOSX (appName))
#elif defined (UNIX)
    impl_ (new SettingsImplUnix (appName))
#endif
{
}

Settings::~Settings ()
{
    impl_.reset ();
}

int
Settings::getInteger (const std::string &section, const std::string &name,
        int defaultValue) const
{
    return impl_->getInteger (section, name, defaultValue);
}

void
Settings::setInteger (const std::string &section, const std::string &name,
        int value)
{
    impl_->setInteger (section, name, value);
}
